extern crate reqwest;
extern crate select;
use select::document::Document;
use select::predicate::{Attr, Class, Name, Predicate};
use std::fs;
use std::fs::File;
use std::fs::OpenOptions;
use std::io::prelude::*;
use std::path::Path;
use std::time::{SystemTime, UNIX_EPOCH};

const TMP_ADDS: &str = "tmp_adds.csv";

fn main() {
  if Path::new(TMP_ADDS).exists() {
    fs::remove_file(TMP_ADDS).unwrap();
  }
  File::create(TMP_ADDS).unwrap();
  leetx();
  skytorrents();
}

fn skytorrents() {
  let page_limit = 100;

  let base_url = "https://www.skytorrents.lol";

  let mut pages: Vec<String> = Vec::new();

  for i in 1..page_limit {
    let page = format!("{}/top100?page={}", base_url, i);
    pages.push(page);
  }

  let types = [
    "video", "audio", "games", "software", "doc", "ebook", "yify", "epub", "xxx", "show", "album",
    "1080",
  ];
  for c_type in types.iter() {
    for i in 1..page_limit {
      let page = format!("{}/top100?type={}&page={}", base_url, c_type, i);
      pages.push(page);
    }
  }

  for page in pages.iter() {
    println!("Fetching page {}", page);
    let html = fetch_html(page);
    let document = Document::from(&html[..]);
    for row in document.find(Attr("id", "results").descendant(Name("tr"))) {
      let info_hash_td = match row.find(Name("td").descendant(Name("a"))).nth(2) {
        Some(t) => t,
        None => continue,
      };
      let info_hash = match info_hash_td.attr("href") {
        Some(t) => t.chars().skip(20).take(40).collect(),
        None => continue,
      };

      let name = match row.find(Name("td").descendant(Name("a"))).nth(0) {
        Some(t) => t.text(),
        None => continue,
      };

      let size = match row.find(Name("td")).nth(1) {
        Some(t) => t.text(),
        None => continue,
      };
      let size_bytes = convert_to_bytes(size);

      // simulate a start and scraped date
      let created_unix = SystemTime::now()
        .duration_since(UNIX_EPOCH)
        .expect("Time went backwards")
        .as_secs();

      let seeders = match row.find(Name("td")).nth(4) {
        Some(t) => t.text().replace(",", ""),
        None => continue,
      };
      let leechers = match row.find(Name("td")).nth(5) {
        Some(t) => t.text().replace(",", ""),
        None => continue,
      };
      // infohash;name;size_bytes;created_unix;seeders;leechers;completed;scraped_date

      let add_line = [
        info_hash,
        name,
        size_bytes.to_string(),
        created_unix.to_string(),
        seeders,
        leechers,
        "".to_string(),
        created_unix.to_string(),
      ].join(";");

      append_to_file(add_line);
    }
  }
}

fn leetx() {
  let page_limit = 50;

  let base_url = "https://1337x.to";

  let mut pages: Vec<String> = Vec::new();

  let types = [
    "Games",
    "Anime",
    "Apps",
    "Documentaries",
    "Movies",
    "Music",
    "Other",
    "TV",
    "XXX",
  ];

  for i in 1..page_limit {
    for c_type in types.iter() {
      let page = format!("{}/sort-cat/{}/seeders/desc/{}/", base_url, c_type, i);
      pages.push(page);
    }
  }

  for page in pages.iter() {
    println!("Fetching page {}", page);
    let html = fetch_html(page);
    let document = Document::from(&html[..]);

    for row in document.find(
      Class("table-list")
        .descendant(Name("tbody"))
        .descendant(Name("tr")),
    ) {
      let detail_page_url_col = match row.find(Class("coll-1")).nth(0) {
        Some(t) => t,
        None => continue,
      };
      let detail_page_url_name = match detail_page_url_col.find(Name("a")).nth(1) {
        Some(t) => t,
        None => continue,
      };
      let detail_page_url_href = match detail_page_url_name.attr("href") {
        Some(t) => t,
        None => continue,
      };

      let detail_full_url = format!("{}{}", base_url, detail_page_url_href);
      println!("Fetching page {}", detail_full_url);
      let detail_html = fetch_html(&detail_full_url);
      let detail_document = Document::from(&detail_html[..]);
      let info_hash = match detail_document
        .find(Class("infohash-box").descendant(Name("span")))
        .nth(0)
      {
        Some(t) => t.text().to_lowercase(),
        None => continue,
      };

      let name_col = match row.find(Class("coll-1")).nth(0) {
        Some(t) => t,
        None => continue,
      };
      let name = match name_col.find(Name("a")).nth(1) {
        Some(t) => t.text(),
        None => continue,
      };

      let seeders = match row.find(Class("coll-2")).nth(0) {
        Some(t) => t.text(),
        None => continue,
      };
      let leechers = match row.find(Class("coll-3")).nth(0) {
        Some(t) => t.text(),
        None => continue,
      };

      let size_col = match row.find(Class("coll-4")).nth(0) {
        Some(t) => t,
        None => continue,
      };
      let size = match size_col.children().nth(0) {
        Some(t) => t.text().replace(",", ""),
        None => continue,
      };

      let size_bytes = convert_to_bytes(size);

      let created_unix = SystemTime::now()
        .duration_since(UNIX_EPOCH)
        .expect("Time went backwards")
        .as_secs();

      let add_line = [
        info_hash,
        name,
        size_bytes.to_string(),
        created_unix.to_string(),
        seeders,
        leechers,
        "".to_string(),
        created_unix.to_string(),
      ].join(";");

      append_to_file(add_line);
    }
  }
}

fn fetch_html(url: &str) -> String {
  reqwest::get(url).unwrap().text().unwrap()
}

fn convert_to_bytes(human: String) -> u64 {
  let split = human.split(" ").collect::<Vec<&str>>();
  let float: f32 = split[0].parse().unwrap();
  let unit = split[1];

  let mut num = (float * 100 as f32) as u64;

  match unit {
    "KB" => num *= 10u64.pow(3),
    "MB" => num *= 10u64.pow(6),
    "GB" => num *= 10u64.pow(9),
    "TB" => num *= 10u64.pow(12),
    "PB" => num *= 10u64.pow(15),
    _ => println!("Unknown unit"),
  }
  num /= 100;
  num
}

fn append_to_file(line: String) {
  let mut file = OpenOptions::new()
    .write(true)
    .append(true)
    .open(TMP_ADDS)
    .unwrap();

  if let Err(e) = writeln!(file, "{}", line) {
    eprintln!("Couldn't write to file: {}", e);
  }
}
